﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Lucene.Net.Search;
using Lucene.Net.Store;
using Lucene.Net.Index;
using Lucene.Net.Documents;
using Lucene.Net.Search.Highlight;
using System.Configuration;
using Ingeniux.CMS;
using Ingeniux.Search;
using MoreLinq;

namespace Ingeniux.Runtime.Controllers
{
	public class IngeniuxController : CMSPageDefaultController
	{
		internal override ActionResult handleStandardCMSPageRequest(CMSPageRequest pageRequest)
		{
			if (pageRequest.RootElementName == ConfigurationManager.AppSettings["LuceneSearchSchema"])
			{
			    var terms = pageRequest.QueryString["term"];
                var filters = pageRequest.QueryString["filters"];
                var catids = pageRequest.QueryString["catids"];
                var types = pageRequest.QueryString["types"];

				pageRequest.Tag = getSearchResults(terms, filters, catids, types);
			}
			return base.handleStandardCMSPageRequest(pageRequest);
		}

		private SearchResults getSearchResults(string terms, string filters, string catids, string types, string page = null, string site = null, int pagesize = 10, int fragsize = 200)
		{
            string[] termsA = terms.ToNullOrWhitespaceHelper()
                .Propagate(
                    s => s.Split(','))
                .Return(new string[0]);

            string[] catsA = catids.ToNullOrWhitespaceHelper()
                .Propagate(
                    s => s.Split(','))
                .Return(new string[0]);

            string[] typesA = types.ToNullOrWhitespaceHelper()
                .Propagate(
                    s => s.Split(','))
                .Return(new string[0]);

            // Add filters
		    if (!string.IsNullOrWhiteSpace(filters))
		    {
		        catsA = filters.Split(',').Concat(catsA).ToArray();
		    }

            bool all = false;
            int p = 0;

            /* You can see in the calling method that we are not attempting to retrieve the page from the request. It could also be retrieved from the Request[""] collection. 
             * If all the results are needed for displaying counts of filters or other such actions you can pass true directly to the Query() method below. In this example
             * since we are not passing a page number in it will default to true. */
            if (!int.TryParse(page, out p))
                all = true;
            else if (p < 1)
                all = true;

            /* SearchResults provides a convenient class for organizing information. This can be substituted out by adding things to the ViewBag.
             * If this is the case update the view to reflect the different type for the model. Feel free to exclude any of these values as well.
             */
            SearchResults result = new SearchResults
            {
                Query = new SearchQuery
                {
                    ProfileName = site,
                    Terms = termsA,
                    Categories = catsA,
                    Types = typesA,
                    All = all,
                    Page = p,
                    PageSize = pagesize,
                    BaseMatchFragmentSize = fragsize
                }
            };

            try
            {
                /* The Search class' default constructor will use the highlighter tags defined in the configuration. Alternatively, a new
                 * highlighter object can be defined and passed in.
                 */
                Search.Search search = new Search.Search();

                int count = 0;
                var siteSearch = Reference.Reference.SiteSearch;
                if (siteSearch != null)
                {
                    /* The Search class has a couple different overloads for the Query() method. Here we are using QueryFinal() method.
                     * QueryFinal() will work for most users but if more control is needed use one of the Query() methods.
                     */
                    IEnumerable<SearchResultItem> results =
                        search.QueryFinal(siteSearch, out count, site, termsA, catsA, typesA, fragsize, all, p, pagesize);

                    result.Items = results.ToList();
                }
                result.ResultsCount = count;
            }
            catch (Exception e)
            {
                //this is a recursive constructor that will capture all inner exception layers
                result.Error = new ErrorResult(e);
            }

            return result;
		}
	}
}
